@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018, 2019, 2020 Esteban J. G. Gabancho.
+# Copyright (C) 2020 CESNET
 #
 # oarepo-s3 is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -51,11 +51,14 @@ from invenio_files_rest.signals import file_deleted
 from invenio_files_rest.tasks import remove_file_data
 from invenio_files_rest.views import check_permission
 from invenio_records_rest.views import pass_record
+from invenio_rest import csrf
 from webargs import fields
 from webargs.flaskparser import use_kwargs
 
 from oarepo_s3.constants import MULTIPART_CONFIG_TAG, MULTIPART_EXPIRATION_TAG
 from oarepo_s3.proxies import current_s3
+from oarepo_s3.signals import after_upload_complete, before_upload_complete, before_upload_abort, \
+    after_upload_abort
 
 multipart_complete_args = {
     'parts': fields.List(
@@ -68,6 +71,7 @@ def pass_file_rec(f):
     """Decorator to retrieve a FileObject
        given by key from record FilesIterator.
     """
+
     @wraps(f)
     def inner(self, pid, record, key, *args, **kwargs):
         files = record.files
@@ -77,6 +81,7 @@ def pass_file_rec(f):
             abort(404, 'upload not found')
         return f(self, pid=pid, record=record, key=key,
                  files=files, file_rec=file_rec, *args, **kwargs)
+
     return inner
 
 
@@ -84,6 +89,7 @@ def pass_multipart_config(f):
     """Decorator to retrieve a multipart upload
        configuration from FileObject tags.
     """
+
     @wraps(f)
     def inner(self, pid, record, key, files, file_rec, *args, **kwargs):
         mc = file_rec.obj.get_tags().get(MULTIPART_CONFIG_TAG, None)
@@ -93,6 +99,7 @@ def pass_multipart_config(f):
         mc = json.loads(mc)
         return f(self, pid=pid, record=record, key=key, files=files,
                  file_rec=file_rec, multipart_config=mc, *args, **kwargs)
+
     return inner
 
 
@@ -127,6 +134,12 @@ class MultipartUploadCompleteResource(MethodView):
     @pass_multipart_config
     @use_kwargs(multipart_complete_args)
     def post(self, pid, record, key, files, file_rec, multipart_config, parts):
+        before_upload_complete.send(file_rec,
+                                    record=record,
+                                    file=file_rec,
+                                    parts=parts,
+                                    multipart_config=multipart_config)
+
         res = current_s3.client.complete_multipart_upload(
             multipart_config['bucket'],
             multipart_config['key'],
@@ -140,12 +153,14 @@ class MultipartUploadCompleteResource(MethodView):
             etag = 'etag:{}'.format(res['ETag'])
             file_rec.obj.file.checksum = etag
             file_rec['checksum'] = etag
+
+            after_upload_complete.send(file_rec, record=record, file=file_rec, files=files)
+
             files.flush()
             record.commit()
 
         db.session.commit()
-
-        return jsonify(res)
+        return jsonify(file_rec.dumps())
 
 
 class MultipartUploadAbortResource(MethodView):
@@ -156,6 +171,10 @@ class MultipartUploadAbortResource(MethodView):
     @pass_file_rec
     @pass_multipart_config
     def post(self, pid, record, files, file_rec, multipart_config, key):
+        before_upload_abort.send(file_rec,
+                                 record=record,
+                                 file=file_rec,
+                                 multipart_config=multipart_config)
 
         res = current_s3.client.abort_multipart_upload(
             multipart_config['bucket'],
@@ -173,6 +192,8 @@ class MultipartUploadAbortResource(MethodView):
 
         db.session.commit()
 
+        after_upload_abort.send(file_rec, record=record, file=file_rec)
+
         return jsonify(res)
 
 
@@ -182,11 +203,11 @@ def multipart_actions(code, files, rest_endpoint, extra, is_draft):
     # rest path -> view
     return {
         'files/<key>/complete-multipart':
-            MultipartUploadCompleteResource.as_view(
+            csrf.exempt(MultipartUploadCompleteResource.as_view(
                 MultipartUploadCompleteResource.view_name.format(endpoint=code)
-            ),
+            )),
         'files/<key>/abort-multipart':
-            MultipartUploadAbortResource.as_view(
+            csrf.exempt(MultipartUploadAbortResource.as_view(
                 MultipartUploadAbortResource.view_name.format(endpoint=code)
-            )
+            ))
     }
